# $Id: isa_filehandling.tcl,v 1.4 2002/07/04 07:55:24 rrp Exp $	

# Procedure opens file
proc OpenFile {filename} {
    global PolyData
    global maintitle
    
    if {$filename==""} {
	# Use the graphics menu
	puts "Trying filedialog"
	set types {
	    {{Visualization Toolkit (polygonal)}    {.vtk}        }
	    {{Stereo-Lithography}                   {.stl}        }
	    {{All Files }                           *             }
	}
	puts "getting filename"
	set filename [tk_getOpenFile -filetypes $types]
	puts "got filename"
    } else {
	if {![file readable $filename]} {
	    tk_messageBox -icon error -title "Error loading file" \
		-message "Error: file '$filename' is not readable!"
	    set filename ""
	    
	}
    }
    if { $filename != "" } {
	
	if { [info commands reader] != "" } {reader Delete}
	switch -glob $filename {
	    *.vtk  {
		vtkPolyDataReader reader
		reader SetFileName $filename
	    } 
	    *.stl {
		vtkSTLReader reader
		reader SetFileName $filename
	    }
	    default {
		vtkPolyDataReader reader
		reader SetFileName $filename
	    }	
	}
	reader Update
	
	PolyData CopyStructure [reader GetOutput]
	[PolyData GetPointData] PassData [[reader GetOutput] GetPointData]
	PolyData Modified
	
	StatusMessage "loaded object '$filename'"

	wm title . "$maintitle - \[$filename\]"
	UpdateGUI

	reader Delete
    }
}

#Procedure saves plane to file
proc SavePlane {} {
    set types {
        {{Visualization Toolkit (polygonal)}    {.vtk}        }
        {{All Files }                           *             }
    }
    set filename [tk_getSaveFile -filetypes $types]
    if { $filename != "" } {
        if { [info commands writer] != "" } {writer Delete}
	switch -glob $filename {
	    *.vtk {
		vtkPolyDataWriter writer
		writer SetFileName $filename
	    } 
	    default {
		# save file as a VTK-file. Add '.vtk' to filename
		vtkPolyDataWriter writer
		writer SetFileName $filename.vtk
	    }
	}
	puts "writing plane"
	
	vtkPoints pts
	vtkCellArray verts
	vtkPolyData pd
	
	set tpoint [planeWidget GetOrigin]
#	puts "temp point: $tpoint"
	set tid [pts InsertNextPoint [lindex $tpoint 0] [lindex $tpoint 1] [lindex $tpoint 2]]
	verts InsertNextCell 1
	verts InsertCellPoint $tid

	set tpoint [planeWidget GetPoint1]
#	puts "temp point: $tpoint"
	set tid [pts InsertNextPoint [lindex $tpoint 0] [lindex $tpoint 1] [lindex $tpoint 2]]
	verts InsertNextCell 1
	verts InsertCellPoint $tid

	set tpoint [planeWidget GetPoint2]
#	puts "temp point: $tpoint"
	set tid [pts InsertNextPoint [lindex $tpoint 0] [lindex $tpoint 1] [lindex $tpoint 2]]
	verts InsertNextCell 1
	verts InsertCellPoint $tid

	pd SetPoints pts
	pd SetVerts verts
	writer SetInput pd
	writer Write

	StatusMessage "Saved plane to file"

	writer Delete
    }
}

#Procedure cut profile to a file
proc SaveCutProfile {} {
    set types {
        {{Visualization Toolkit (polygonal)}    {.vtk}        }
        {{All Files }                           *             }
    }
    set filename [tk_getSaveFile -filetypes $types]
    if { $filename != "" } {
        if { [info commands writer] != "" } {writer Delete}
	switch -glob $filename {
	    *.vtk {
		vtkPolyDataWriter writer
		writer SetFileName $filename
	    } 
	    default {
		# save file as a VTK-file. Add '.vtk' to filename
		vtkPolyDataWriter writer
		writer SetFileName $filename.vtk
	    }
	}
	writer SetInput CutPart
	writer Write

	StatusMessage "Saved cutted profile to file $filename"
	writer Delete
    }
}


proc ReadPlaneFromFile { filename } {
    
    # Security delete
    if { [info commands reader] != "" } {reader Delete}
    
    vtkPolyDataReader reader
    reader SetFileName $filename
    reader Update
    
    set max [[reader GetOutput] GetNumberOfPoints]
    
    if {$max != 3} {
	StatusMessage "Error: Not 3 points in file ($max found)"	
    } else {
	set porigin [[reader GetOutput] GetPoint 0]
	set ppoint1 [[reader GetOutput] GetPoint 1]
	set ppoint2 [[reader GetOutput] GetPoint 2]
	#	puts "Points: ($porigin) ($ppoint1) ($ppoint2)"

	planeWidget SetOrigin [lindex $porigin 0] [lindex $porigin 1] [lindex $porigin 2]
	planeWidget SetPoint1 [lindex $ppoint1 0] [lindex $ppoint1 1] [lindex $ppoint1 2]
	planeWidget SetPoint2 [lindex $ppoint2 0] [lindex $ppoint2 1] [lindex $ppoint2 2]
	StatusMessage "Read plane from $filename"
    }
    reader Delete
}


# Read plane from a file
proc OpenPlaneFile {filename} {
    if {$filename==""} {
	# Use the graphics menu
	set types {
	    {{VTK file}	      {.vtk}   }
            {{All Files }              *       }
	}
	set filename [tk_getOpenFile -filetypes $types]
    } else {
	if {![file readable $filename]} {
	    tk_messageBox -icon error -title "Error loading file" \
		-message "Error: file '$filename' is not readable!"
            set filename ""
	}
    }
    if { $filename != "" } {
	switch -glob $filename {
	    *.vtk {
		ReadPlaneFromFile $filename
	    }
	    default {
		StatusMessage "Unrecognised file extension"
	    }
	}
    }
}


# Read landmarks from a file
proc OpenLMPointFile {filename} {
    if {$filename==""} {
	# Use the graphics menu
	set types {
	    {{VTK point cloud}	      {.vtk}   }
	    {{Landmark Point File}    {.lmp}   }
            {{All Files }              *       }
	}
	set filename [tk_getOpenFile -filetypes $types]
    } else {
	if {![file readable $filename]} {
	    tk_messageBox -icon error -title "Error loading file" \
		-message "Error: file '$filename' is not readable!"
            set filename ""
	}
    }
    if { $filename != "" } {
	switch -glob $filename {
	    *.lmp {
		if [catch {open $filename r} fileID] {
		    StatusMessage "Could not read points from $filename"
		} else {
		    LMP_ReadPoints $fileID
		    close $fileID
		    StatusMessage "Read points from $filename"
		}
	    }
	    *.vtk {
		LMP_ReadPointsInVTKFormat $filename
		StatusMessage "Read points from $filename in VTK format"
	    }
	    default {
		StatusMessage "Unrecognised file extension"
	    }
	}
    }
}

# Save landmarks to a file
proc SaveLMPointFile {} {
    # Use the graphics menu
    set types {
	{{VTK point cloud}	   {.vtk}	 }
	{{Landmark Point File}	   {.lmp}	 }
	{{All Files }		   *		 }
    }
    set filename [tk_getSaveFile -filetypes $types]
    
    if { $filename != "" } {
	switch -glob $filename {
	    *.lmp {
		if [catch {open $filename w} fileID] {
		    StatusMessage "Could not save points to $filename"
		} else {
		    LMP_WritePoints $fileID
		    close $fileID
		    StatusMessage "Saved points to $filename"
		}
	    }
	    *.vtk {
		LMP_WritePointsInVTKFormat $filename
		StatusMessage "Wrote landmarks in VTK point format in $filename"
	    } 
	    default {
		StatusMessage "Unrecognised file extension"
	    }
	}
    }
}
